package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.TCStatisticsDto;
import com.qmth.distributed.print.business.entity.TCStatistics;
import com.qmth.distributed.print.business.entity.TCStatisticsTemp;
import com.qmth.distributed.print.business.enums.StatisticsStatusEnum;
import com.qmth.distributed.print.business.mapper.TCStatisticsMapper;
import com.qmth.distributed.print.business.service.TCStatisticsService;
import com.qmth.distributed.print.business.service.TCStatisticsTempService;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.util.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题统计表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-09-09
 */
@Service
public class TCStatisticsServiceImpl extends ServiceImpl<TCStatisticsMapper, TCStatistics> implements TCStatisticsService {

    @Resource
    TCStatisticsMapper tcStatisticsMapper;

    @Resource
    TCStatisticsTempService tcStatisticsTempService;

    @Resource
    SysOrgService sysOrgService;

    @Resource
    RedisUtil redisUtil;

    /**
     * 查找印刷计划信息
     *
     * @param iPage
     * @param collegeId
     * @param teachingRoomId
     * @param status
     * @param courseName
     * @param teacherName
     * @param schoolId
     * @param userId
     * @return
     */
    @Override
    public IPage<TCStatisticsDto> list(IPage<Map> iPage, Long collegeId, Long teachingRoomId, StatisticsStatusEnum status, String courseName, String teacherName, Long schoolId, Long userId) {
        return tcStatisticsMapper.list(iPage, collegeId, teachingRoomId, Objects.nonNull(status) ? status.name() : null, courseName, teacherName, schoolId, userId);
    }

    /**
     * 导入关联数据
     *
     * @param sysUser
     * @param batchNo
     */
    @Override
    @Transactional
    public void importJoinData(SysUser sysUser, String batchNo) {
        List<TCStatisticsTemp> tcStatisticsTempList = tcStatisticsTempService.findByBatchNo(sysUser.getSchoolId(), batchNo);
        if (Objects.nonNull(tcStatisticsTempList) && tcStatisticsTempList.size() > 0) {
            batchNo = SystemConstant.getUuid();
            Set<Long> orgIds = this.joinDataGetOrgIds(sysUser, batchNo, tcStatisticsTempList);
            List<TCStatisticsDto> tcStatisticsDtoList = this.findByBatchNoCount(sysUser.getSchoolId(), batchNo, orgIds);
            if (Objects.nonNull(tcStatisticsDtoList) && tcStatisticsDtoList.size() > 0) {
                this.saveJoinData(sysUser, tcStatisticsDtoList);
            }
        }
    }

    /**
     * 删除导入数据
     *
     * @param userId
     * @param courseSet
     * @param setCollections
     */
    @Override
    @Transactional
    public void removeImportData(Long userId, Set<String> courseSet, Set<Long>... setCollections) {
        TCStatisticsService tcStatisticsService = SpringContextHolder.getBean(TCStatisticsService.class);
        QueryWrapper<TCStatistics> tcStatisticsQueryWrapper = new QueryWrapper<>();
        tcStatisticsQueryWrapper.lambda().in(TCStatistics::getCollegeId, setCollections[0])
                .in(TCStatistics::getTeachingRoomId, setCollections[1])
                .in(TCStatistics::getCourseCode, courseSet)
                .in(TCStatistics::getClazzId, setCollections[2])
                .eq(TCStatistics::getCreateId, userId);
        tcStatisticsService.remove(tcStatisticsQueryWrapper);

        QueryWrapper<TCStatisticsTemp> tcStatisticsTempQueryWrapper = new QueryWrapper<>();
        tcStatisticsTempQueryWrapper.lambda().in(TCStatisticsTemp::getCollegeId, setCollections[0])
                .in(TCStatisticsTemp::getTeachingRoomId, setCollections[1])
                .in(TCStatisticsTemp::getCourseCode, courseSet)
                .in(TCStatisticsTemp::getClazzId, setCollections[2])
                .eq(TCStatisticsTemp::getCreateId, userId);
        tcStatisticsTempService.remove(tcStatisticsTempQueryWrapper);
    }

    /**
     * 刷新数据
     *
     * @return
     */
    @Override
    @Transactional
    public Result freshenData() {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TCStatistics> tcStatisticsQueryWrapper = new QueryWrapper<>();
        tcStatisticsQueryWrapper.select(" DISTINCT batch_no ").eq("create_id", sysUser.getId());
        List<TCStatistics> tcStatisticsList = this.list(tcStatisticsQueryWrapper);
        if (Objects.nonNull(tcStatisticsList) && tcStatisticsList.size() > 0) {
            Set<String> batchNoSet = tcStatisticsList.stream().map(s -> s.getBatchNo()).collect(Collectors.toSet());
            boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_BATCH_NO_PREFIX + Math.abs(batchNoSet.toString().hashCode()), SystemConstant.REDIS_LOCK_BATCH_NO_TIME_OUT);
            if (!lock) {
                throw ExceptionResultEnum.ERROR.exception("正在刷新数据，请稍候再试！");
            }
            try {
                this.freshenJoinData(sysUser, batchNoSet);
            } catch (Exception e) {
                log.error(SystemConstant.LOG_ERROR, e);
                if (e instanceof ApiException) {
                    return ResultUtil.error((ApiException) e, e.getMessage());
                } else {
                    return ResultUtil.error(e.getMessage());
                }
            } finally {
                redisUtil.releaseLock(SystemConstant.REDIS_LOCK_BATCH_NO_PREFIX + Math.abs(batchNoSet.toString().hashCode()));
            }
        }
        return ResultUtil.ok(true);
    }

    /**
     * 删除数据
     *
     * @param ids
     * @return
     */
    @Override
    @Transactional
    public Result deleteData(List<Long> ids) {
        TCStatisticsService tcStatisticsService = SpringContextHolder.getBean(TCStatisticsService.class);
        tcStatisticsService.removeByIds(ids);
        return ResultUtil.ok(true);
    }

    /**
     * 关联数据获取orgIds
     *
     * @param sysUser
     * @param batchNo
     * @param tcStatisticsTempList
     * @return
     */
    private Set<Long> joinDataGetOrgIds(SysUser sysUser, String batchNo, List<TCStatisticsTemp> tcStatisticsTempList) {
        Set<Long> collegeIdSet = new HashSet<>();
        for (TCStatisticsTemp t : tcStatisticsTempList) {
            collegeIdSet.add(t.getCollegeId());
            t.insertInfo(sysUser.getId());
            t.setBatchNo(batchNo);
        }
        tcStatisticsTempService.saveBatch(tcStatisticsTempList);

        Set<Long> orgIds = new HashSet<>();
        for (Long l : collegeIdSet) {
            List<SysOrg> sysOrgList = sysOrgService.findByConnectByRootOrgId(l);
            Set<Long> orgTempIds = sysOrgList.stream().map(s -> s.getId()).collect(Collectors.toSet());
            orgIds.addAll(orgTempIds);
        }
        return orgIds;
    }

    /**
     * 保存正式统计数据
     *
     * @param sysUser
     * @param tcStatisticsDtoList
     */
    private void saveJoinData(SysUser sysUser, List<TCStatisticsDto> tcStatisticsDtoList) {
        //进行过滤
        Map<String, TCStatisticsDto> map = new LinkedHashMap<>();
        for (TCStatisticsDto t : tcStatisticsDtoList) {
            String key = t.getCollegeId() + ":" +
                    t.getTeachingRoomId() + ":" +
                    t.getCourseCode() + ":" +
                    t.getPaperNumber() + ":" +
                    t.getPrintPlanId() + ":" +
                    t.getClazzId();
            if (!map.containsKey(key)) {
                map.computeIfAbsent(key, v -> t);
            } else {
                TCStatisticsDto temp = map.get(key);
                if (t.getPrintSum().intValue() > temp.getPrintSum().intValue()) {
                    map.put(key, t);
                } else if (t.getPrintSum().intValue() == temp.getPrintSum().intValue()) {
                    if (t.getStatus() != StatisticsStatusEnum.UN_JOIN) {
                        temp.setExamDetailId(t.getExamDetailId());
                    } else {
                        temp.setExamDetailId(null);
                    }
                }
            }
        }
        List<TCStatistics> tcStatisticsList = new Gson().fromJson(JacksonUtil.parseJson(map.values()), new TypeToken<List<TCStatistics>>() {
        }.getType());
        for (TCStatistics t : tcStatisticsList) {
            t.insertInfo(sysUser.getId());
        }
        TCStatisticsService tcStatisticsService = SpringContextHolder.getBean(TCStatisticsService.class);
        tcStatisticsService.saveBatch(tcStatisticsList);
    }

    /**
     * 刷新关联数据
     *
     * @param sysUser
     * @param batchNoSet
     */
    public void freshenJoinData(SysUser sysUser, Set<String> batchNoSet) {
        TCStatisticsService tcStatisticsService = SpringContextHolder.getBean(TCStatisticsService.class);
        for (String batchNo : batchNoSet) {
            List<TCStatisticsTemp> tcStatisticsTempList = tcStatisticsTempService.findByBatchNoJoin(sysUser.getSchoolId(), batchNo);
            if (Objects.nonNull(tcStatisticsTempList) && tcStatisticsTempList.size() > 0) {
                QueryWrapper<TCStatisticsTemp> tcStatisticsTempQueryWrapper = new QueryWrapper<>();
                tcStatisticsTempQueryWrapper.lambda().eq(TCStatisticsTemp::getBatchNo, batchNo);
                tcStatisticsTempService.remove(tcStatisticsTempQueryWrapper);

                String batchNoNew = SystemConstant.getUuid();
                Set<Long> orgIds = this.joinDataGetOrgIds(sysUser, batchNoNew, tcStatisticsTempList);
                List<TCStatisticsDto> tcStatisticsDtoList = this.findByBatchNoCountJoin(sysUser.getSchoolId(), batchNoNew, orgIds);
                if (Objects.nonNull(tcStatisticsDtoList) && tcStatisticsDtoList.size() > 0) {
                    QueryWrapper<TCStatistics> tcStatisticsQueryWrapper = new QueryWrapper<>();
                    tcStatisticsQueryWrapper.lambda().eq(TCStatistics::getBatchNo, batchNo);
                    tcStatisticsService.remove(tcStatisticsQueryWrapper);
                    this.saveJoinData(sysUser, tcStatisticsDtoList);
                }
            }
        }
    }

    /**
     * 根据batchNo统计信息
     *
     * @param schoolId
     * @param batchNo
     * @param orgIds
     * @return
     */
    public List<TCStatisticsDto> findByBatchNoCount(Long schoolId, String batchNo, Set<Long> orgIds) {
        return tcStatisticsMapper.findByBatchNoCount(schoolId, batchNo, orgIds);
    }

    /**
     * 根据batchNo统计信息
     *
     * @param schoolId
     * @param batchNo
     * @param orgIds
     * @return
     */
    public List<TCStatisticsDto> findByBatchNoCountJoin(Long schoolId, String batchNo, Set<Long> orgIds) {
        return tcStatisticsMapper.findByBatchNoCountJoin(schoolId, batchNo, orgIds);
    }
}
