package com.qmth.teachcloud.common.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.common.bean.dto.BlurryUserDto;
import com.qmth.teachcloud.common.bean.dto.LoginDto;
import com.qmth.teachcloud.common.bean.dto.UserDto;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.bean.params.UserPushParam;
import com.qmth.teachcloud.common.bean.params.UserSaveParams;
import com.qmth.teachcloud.common.bean.result.LoginResult;
import com.qmth.teachcloud.common.bean.result.SyncCountResult;
import com.qmth.teachcloud.common.bean.result.UserRoleNameResult;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.FlowApproveNameEnum;
import com.qmth.teachcloud.common.enums.userPush.SpecialPrivilegeEnum;
import org.springframework.util.LinkedMultiValueMap;

import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 用户表 服务类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
public interface SysUserService extends IService<SysUser> {

    IPage<UserDto> list(String loginName, String roleId, Boolean enable, String realName, Integer pageNumber, Integer pageSize);

    boolean saveUser(UserSaveParams userSaveParams) throws IllegalAccessException;

    boolean enable(SysUser user) throws NoSuchAlgorithmException, IllegalAccessException;

    boolean resetPassword(Long id) throws NoSuchAlgorithmException;

    /**
     * 更新用户登录信息（密码、手机号）
     * <p>
     * 1.判断用户在系统中是否存在
     * 2.判断用户原密码是否正确
     * 3.更新密码 -> 新旧密码不一致判断
     * 4.更新手机 -> 检验验证码正确新（不为空、正确、未过期）、手机号唯一性校验
     * 5.若用户已经登陆且进行了密码或手机号修改 -> 清空用户登录信息
     * 6.如果不是共用验证码则过期该用户验证码
     * 7.调用用户登录公共方法（验证码参数使用系统共用验证码）登录用户
     * </p>
     *
     * @param user 用户信息
     * @return 登录信息
     * @throws NoSuchAlgorithmException 异常
     */
    LoginResult updatePassword(SysUser user) throws NoSuchAlgorithmException;

    boolean bindRoles(SysUser sysUser);

    LoginDto getUserRoles(SysUser sysUser);

    List<BlurryUserDto> listUser(String courseCode, String param);

    List<SysUser> listByOrgId(Long id);

    boolean saveCustomer(UserSaveParams userSaveParams);

    IPage<UserDto> listCustomer(String realName, Boolean enable, Integer pageNumber, Integer pageSize);

    /**
     * 根据外键查找教师信息
     *
     * @param schoolId    学校id
     * @param teacherName 教师名称
     * @return 学院数据
     */
    SysUser findByForeignKey(Long schoolId, String teacherName);

    /**
     * 临时-不鉴权保存用户 （默认创建用户角色为该学校管理员）
     *
     * @param userSaveParams 用户参数
     * @return 结果
     */
    boolean saveUserNoAuth(UserSaveParams userSaveParams);

    /**
     * 过滤拥有审核权限的人
     *
     * @param schoolId
     * @param orgIds
     * @param filter
     * @param approveName
     * @param realName
     * @return
     */
    List<ApproveUserResult> filterFlowPrivilege(Long schoolId, Set<Long> orgIds, Boolean filter, FlowApproveNameEnum approveName, String realName);

    /**
     * 执行批量导入用户逻辑
     *
     * @param finalList 基础课程数据集合
     * @param map       参数
     * @return 结果
     */
    Map<String, Object> executeSysUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) throws NoSuchAlgorithmException;

    /**
     * 获取命题审批人
     *
     * @param courseCode
     * @param realName
     * @return
     */
    public List<ApproveUserResult> findByAssignApprove(String courseCode, String realName);

    /**
     * 获取命题审批人
     *
     * @param realName
     * @return
     */
    public List<ApproveUserResult> findByJxzyyAssignApprove(String realName);

    /**
     * 获取教研室主任审批人
     *
     * @param courseCode
     * @param realName
     * @return
     */
    public List<ApproveUserResult> findByDirectorApprove(String courseCode, String realName);

    /**
     * 获取教研室主任审批人
     *
     * @param realName
     * @param flowApproveNameEnum
     * @return
     */
    public List<ApproveUserResult> findByJxzyyDirectorApprove(String realName, FlowApproveNameEnum flowApproveNameEnum);

    /**
     * 获取院长审批人
     *
     * @param orgIds
     * @param realName
     * @return
     */
    public List<ApproveUserResult> findByPresidentApprove(List<Long> orgIds, String realName);

    /**
     * 获取主任或院长审批人
     *
     * @param orgIds
     * @param realName
     * @param flowApproveNameEnum
     * @return
     */
    public List<ApproveUserResult> findByJxzyyTopDirectorOrPresidentApprove(List<Long> orgIds, String realName, FlowApproveNameEnum flowApproveNameEnum);

    /**
     * 获取同级别下一级审批人
     *
     * @param courseCode
     * @param orgIds
     * @param realName
     * @param flowApproveNameEnum
     * @return
     */
    public List<ApproveUserResult> findByRootApprove(String courseCode, List<Long> orgIds, String realName, FlowApproveNameEnum flowApproveNameEnum);

    /**
     * 根据id获取
     *
     * @param id
     * @return
     */
    public List<ApproveUserResult> findById(Long id);

    /**
     * 根据机构id获取
     *
     * @param orgIdSet
     * @return
     */
    public List<ApproveUserResult> findByOrgIds(Set<Long> orgIdSet);

    /**
     * 校验短信验证码
     *
     * @param userId       用户id
     * @param mobileNumber 用户手机号
     * @param code         验证码
     */
    void checkSmsCode(Long userId, String mobileNumber, String code);

    /**
     * 强行过期用户手机号验证码（过期时间改为验证码发送时间）
     *
     * @param userId       用户id
     * @param mobileNumber 用户手机号
     */
    void expiredVerifyCode(Long userId, String mobileNumber);

    /**
     * 获取教研室id底下的人数量
     *
     * @param orgId
     * @return
     */
    public int countByTeachingRoomId(Long orgId);

    /**
     * 用户推送云阅卷服务
     *
     * @param userPushParamList 推送参数
     * @param requestUserId     请求的用户id
     * @throws IllegalAccessException
     */
    boolean userPushService(List<UserPushParam> userPushParamList, Long requestUserId) throws IllegalAccessException;

    /**
     * 自动推送用户推送失败表用户到云阅卷
     */
    void autoPushUserErrorData() throws IllegalAccessException;

    /**
     * 查询用户角色id查询用户特殊权限
     *
     * @param roleIdSet 用户角色id
     * @return 用户特殊权限身份
     */
    SpecialPrivilegeEnum findUserSpecialPrivilegeByRoleIdSet(Set<Long> roleIdSet);

    /**
     * 根据用户id查询当前用户的特殊权限
     *
     * @param userId 用户id
     * @return 用户特殊权限身份
     */
    SpecialPrivilegeEnum findUserSpecialPrivilegeByUserId(Long userId);

    SyncCountResult needPushAgainOperate(SysUser requestUser) throws IllegalAccessException;

    /**
     * 查找用户角色名
     *
     * @param userIds
     * @return
     */
    List<UserRoleNameResult> selectRoleNames(List<Long> userIds);
}
